# -*- coding: utf-8 -*- 
""" 
Created on Wed Aug 24 16:22:39 2016 
 
@author: ppradeep 
""" 
import os
clear = lambda: os.system('cls')
clear()

#%% 
import pandas as pd 
import numpy as np 
import matplotlib.pyplot as plt
from sklearn.feature_selection import VarianceThreshold

path = 'C:/Users/ppradeep/Desktop/HTTK/' 
path = 'Z:/Projects/HTTK/' 

#%%
## User-defined functions
def selectFeatures_VarThresh(X, threshold):
    sel = VarianceThreshold(threshold=(threshold*(1-threshold))) 
    X_sel = sel.fit_transform(X)
    # Convert it into a dataframe 
    x_tr = pd.DataFrame(X_sel, index = X.index) 
    x_tr.columns = X.columns[sel.get_support(indices = True)]
    return x_tr

## Remove culumns with >80% correlation
def correlation(dataset, threshold):
    col_corr = set() # Set of all the names of deleted columns
    corr_matrix = dataset.corr()
    for i in range(len(corr_matrix.columns)):
        for j in range(i):
            if corr_matrix.iloc[i, j] >= threshold:
                colname = corr_matrix.columns[i] # getting the name of column
                col_corr.add(colname)
                if colname in dataset.columns:
                    del dataset[colname] # deleting the column from the dataset
    return dataset

def annotated_boxplot(df, y_var, text):
    upper_quartile = df.quantile(q=0.75)
    lower_quartile = df.quantile(q=0.25)
    iqr = upper_quartile - lower_quartile
    upper_whisker = df[df<=upper_quartile+1.5*iqr].max()
    ax = df_kmeans.boxplot(column = y_var, by = 'Cluster', showfliers = False, figsize=[12,8])
    for x, y in enumerate(upper_whisker):
        ax.annotate(count[idx], (x,y), fontsize = 12)

#%%
###########################################################################
## Read input data 
###########################################################################

data1 = pd.read_csv(path+'data/Prachi-112117.txt', index_col = 'CAS').loc[:,['All.Compound.Names', 'Human.Funbound.plasma', 'Human.Clint']]
data1.rename(columns={'All.Compound.Names' : 'Name'}, inplace = True)
data2 = pd.read_excel(path+'data/AFFINITY_Model_Results-2018-02-27.xlsx', index_col = 'CAS').loc[:,['Name','Fup.Med']]
data2.rename(columns={'Name': 'All.Compound.Names','Fup.Med':'Human.Funbound.plasma'}, inplace = True)
data3 = pd.read_excel(path+'data/CLint-2018-03-01-Results.xlsx', index_col = 'CAS').loc[:,['Name','CLint.1uM.Median']]
data3.rename(columns={'Name': 'All.Compound.Names','CLint.1uM.Median':'Human.Clint'}, inplace = True)


## Select y variable
y_var = 'Human.Funbound.plasma'
y_var = 'Human.Clint'

# Create a new dataframe with chemical names and y variable value based on raw data
casList = list(set(data1.index.tolist()+data2.index.tolist()+data3.index.tolist()))
data = pd.DataFrame(index = casList, columns = ['Name',y_var])

# Update the training data. If y value is available from later data (data 2 or 3) use that, if not use from old data (data1)
for cas in data.index:
    try:
        if cas in data1.index:
            data.loc[cas,'Name'] = data1.loc[cas,'Name']
            data.loc[cas,y_var] = data1.loc[cas,y_var]
        if cas in data2.index:
            data.loc[cas,'Name'] = data2.loc[cas,'Name']
            data.loc[cas,y_var] = data2.loc[cas,y_var]
    except:
        pass
data.dropna(inplace = True) #Retain data with y variable values

## Extract y data
Y = data[y_var]
Y_index = Y.index
#%%
# Transform Y variable
if y_var == 'Human.Clint':
    [Y.set_value(idx, 0.1) for idx in Y[Y < 0.1].index] 
    [Y.set_value(idx, 200) for idx in Y[Y > 100].index] 
    Y_trans = Y.apply(lambda x: np.log10(x))
    Y = pd.Series(Y_trans, index = Y_index) 

if y_var == 'Human.Funbound.plasma':
    Y[Y==1.0] = 0.99
    Y[Y==0] = 0.005
    Y_trans = (1-Y)/Y
    Y_trans = Y_trans.apply(lambda x: np.log10(x))
    Y = pd.Series(Y_trans, index = Y.index) 

Y.hist(bins=20)
plt.xlabel('%s bins' %(y_var), size = 18)
plt.ylabel('Frequency', size = 18)
Y_index = Y.index

#%%
# --------------------------------------------------
## Read fingerprints/descriptors
# --------------------------------------------------
### Chemotyper FPs: 779 Toxprints 
#df_chemotypes = pd.read_csv(path+'data/toxprint.txt', sep = ';', index_col='M_NAME') #Rename 'M_NAME' to 'CAS' in data file
### PubChem FPs: 881 bits
#df_pubchem = pd.read_csv(path+'data/pubchem.txt', index_col='row ID')
## combine fingerprints
#fingerprints = pd.concat([df_pubchem, df_chemotypes], axis=1).dropna()
#
## Remove culumns with >80% correlation and >80% variance threshold
#fingerprints = selectFeatures_VarThresh(fingerprints, 0.80)
#fingerprints = correlation(fingerprints, 0.80)
#
#fingerprints.to_csv(path+'data/CombinedFP_Prachi-112117.txt', index_label='CAS')

fingerprints = pd.read_csv(path+'data/CombinedFP.csv', index_col='row ID') 
# Combine the data
data = pd.concat([fingerprints, Y], axis=1).dropna()
X = data.iloc[:, data.columns != y_var]
Y = data[y_var]

#%%
# --------------------------------------
## 2.	Unsupervised Clustering
# --------------------------------------

####----------------------------
## Choosing K
####----------------------------
from sklearn.cluster import KMeans 
from scipy.spatial.distance import cdist, pdist

k_range = range(5, 55, 5) # Number of clusters
k_means_var = [KMeans(init='k-means++', n_clusters = k).fit(X) for k in k_range]
centroids = [v.cluster_centers_ for v in k_means_var]

k_euclid = [cdist(X, cent, 'euclidean') for cent in centroids]
dist = [np.min(ke, axis=1) for ke in k_euclid]
wcss = [sum(d**2) for d in dist]
tss = sum(pdist(X)**2)/X.shape[0]
bss = tss - wcss
perc_var = bss/tss*100

## Number of clusters chosen kIdx
for i, val in enumerate(perc_var):
    #per_change = 100*(perc_var[i+1] - perc_var[i])/perc_var[i]
    if val > 50:
        kIdx = i
        print kIdx, val
        break
    
#%%
# Graph the variance
import matplotlib.pyplot as plt 
fig = plt.figure(figsize=[12,8], dpi = 300) 
plt.plot(k_range, bss/tss*100, 'r.-', markersize=20) 
plt.plot(k_range[kIdx], bss[kIdx]/tss*100, marker='o', markersize=25, 
    markeredgewidth=2, markeredgecolor='g', markerfacecolor='None')
#plt.ylim((40,80))
plt.xticks(range(5,55,5), fontsize = 24)
plt.yticks(fontsize = 24)
plt.grid(True)
plt.xlabel('Number of Clusters', size = 36, labelpad = 20)
plt.ylabel('Percent Variance Explained', size = 36, labelpad = 20)
plt.tight_layout()
plt.savefig(path+'output/%s_Cluster-Elbow.png' %y_var, bbox_inches='tight')
plt.show() 

#%%
## Implement kmeans clustering 
from sklearn.cluster import KMeans 

k = k_range[kIdx] # Number of clusters
kmeans = KMeans(init='k-means++', n_clusters = k) 
kmeans.fit(X) 
k_means_labels = kmeans.labels_ 
k_means_cluster_centers = kmeans.cluster_centers_ 
k_means_labels_unique = np.unique(k_means_labels) 
x_sel_kmeans = pd.DataFrame(index = X.index)
x_sel_kmeans['Cluster'] = kmeans.predict(X) 
count = x_sel_kmeans['Cluster'].value_counts() #Number of chemicals in each cluster
#%%
## Create a new dataframe where each chemical is represented by a cluster number  
## and its associated Clearance value 
df_kmeans = x_sel_kmeans.copy()
df_kmeans[y_var] = Y 
# Boxplot 
df = df_kmeans
upper_quartile = df.groupby(by='Cluster').quantile(q=0.75)
lower_quartile = df.groupby(by='Cluster').quantile(q=0.25)
iqr = upper_quartile - lower_quartile
ax = df_kmeans.boxplot(column = y_var, by = 'Cluster', showfliers = False, figsize=[14,8])
## Annotate each boxplot with number of chemicals in each cluster
for x in upper_quartile.index:
    ax.annotate(count[x], (x+1,upper_quartile.loc[x,y_var]+0.05), fontsize = 14, color = 'r')
     
plt.title("") 
plt.suptitle("")  
xtick_label = range(1,k+1)
plt.xticks(range(1,k+1),xtick_label, fontsize = 24, rotation = 'vertical')
plt.yticks(fontsize = 24)
plt.ylabel('Transformed Intrinsic \n Clearance',  size = 36, labelpad = 10)
plt.xlabel('Cluster Number',  size = 36, labelpad = 20)
plt.grid(False)
plt.tight_layout()
plt.savefig(path+'output/%s_Cluster-BoxPlot.png'%y_var, bbox_inches='tight')
plt.show() 

#%%
# --------------------------------------
## 3.	T-tests for mean between clusters
# --------------------------------------
## T-test between clusters to calculate independence between means of HClint between clusters     
from scipy.stats import ttest_ind 
import numpy as np
import seaborn as sns

fig = plt.figure(figsize=[12,8], dpi = 300)
p_vals = np.ones((k,k))
for cluster_no in range(0,k): 
    for i in range(cluster_no+1, k): 
        y0 = df_kmeans[df_kmeans['Cluster'] == cluster_no][y_var] 
        y1 = df_kmeans[df_kmeans['Cluster'] == i][y_var] 
        t_val, p_val = ttest_ind(y0, y1) 
        p_vals[cluster_no][i] = p_val; p_vals[i][cluster_no] = p_val
#        print "T-test (cluster %d and %d): p-value = %f" %(cluster_no, i, p_val) 
 
ax = sns.heatmap(p_vals, cmap=plt.cm.Greens) #annot = True
ax.invert_yaxis()
ax.set_xticklabels(range(1,k+1), fontsize = 18, rotation = 'vertical')
ax.set_yticklabels(reversed(range(1, k+1)), fontsize = 18, rotation = 'horizontal')
sns.set(font_scale=2)
#plt.title('T-test: p-value matrix \n (Difference in means between clusters)') 
plt.xlabel('Cluster Number', size = 36, labelpad = 20)
plt.ylabel('Cluster Number', size = 36, labelpad = 20)
plt.tight_layout()
plt.savefig(path+'output/%s_Clusters-Pvalue.png' %y_var, bbox_inches='tight')
plt.show() 

#%%
# --------------------------------------
## 4.	Similarity plots between clusters
# --------------------------------------

# Plot a diversity/cluster plot with each circle as cluster and dots as cluster members and distance
## Cluster plots with distance within clusters 
dist = pd.read_csv(path+'data/CombinedFPDistances.csv', index_col='row ID').fillna(1)
dist.columns = dist.index 

## Select chemicals with data
dist_sel = dist.loc[df_kmeans.index, df_kmeans.index]

## Get the distance matrix for each cluster and plot a distance based heatmap 
import seaborn as sns 
import matplotlib.pyplot as plt

fig = plt.figure(figsize=(k, 17), dpi = 300) #k,k
cbar_ax = fig.add_axes([.91, .3, .03, .4]) 
for i, cluster_no in enumerate(range(0,k)): 
    plot_num = cluster_no + 1
    ax = plt.subplot(int(k/5),5, plot_num)
    cluster_cas = df_kmeans[df_kmeans['Cluster'] == cluster_no].index 
    df_cluster_dist = dist_sel.loc[cluster_cas, cluster_cas] 
    ax = sns.heatmap(df_cluster_dist, ax=ax, 
                cbar = i == 0, cbar_ax = None if i else cbar_ax, 
                vmin=0, vmax=1,
                #cmap=plt.cm.Oranges,
                cmap="Wistia",
                xticklabels= False, yticklabels= False)  

    try:
        cbar = ax.collections[0].colorbar
        cbar.set_ticks([0, 0.25, 0.50, 0.75, 1])
        cbar.set_ticklabels(['0: least \n similar', '0.25', '0.50', '0.75', '1: most \n similar'])   
        cbar.ax.tick_params(labelsize=26) 
    except:
        pass

    ax.set_title('Cluster %d' %(cluster_no+1), fontsize=30)
    n_clusters = len(df_kmeans[df_kmeans['Cluster'] == cluster_no])
    avg_sim = np.mean(df_cluster_dist.values)
    ax.annotate('Chemicals: %d \nAvg. Sim.: %.2f' \
                %(n_clusters, avg_sim), \
                [1, 95*n_clusters/100], fontsize=26)
    ax.axes.get_yaxis().set_visible(False)
    ax.axes.get_xaxis().set_visible(False)
    ax.invert_yaxis()
#sns.set(font_scale=2)    
#fig.text(0.5, 0.06, 'Cluster Number', ha='center', size = 60)
#fig.text(0.06, 0.5, 'Cluster Number', va='center', rotation='vertical', size = 60)
#plt.suptitle('Similarity heatmap by clusters \n (PubChem Fingerprints)') 
plt.savefig(path+'output/%s_ClusteringDistance.png'%y_var, bbox_inches='tight')
