# -*- coding: utf-8 -*-
"""
Created on Fri Feb 09 12:54:17 2018

@author: ppradeep
"""

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import sklearn.metrics as sm
from sklearn.metrics import r2_score

#%%
def boxplot_sorted(df):
    upper_quartile = df.quantile(q=0.75)
    lower_quartile = df.quantile(q=0.25)
    iqr = upper_quartile - lower_quartile
    #upper_whisker = df[df<=upper_quartile+1.5*iqr].max()
    lower_whisker = df[df>=lower_quartile-1.5*iqr].min()
    sorted_index = lower_whisker.sort_values().index
    lower_whisker = lower_whisker.sort_values()
    #return df[sorted_index].boxplot(showfliers=False, return_type="both"), sorted_index, lower_whisker
    return sorted_index, lower_whisker

def calc_lowerwhisker(df):
    upper_quartile = df.quantile(q=0.75)
    lower_quartile = df.quantile(q=0.25)
    iqr = upper_quartile - lower_quartile
    lower_whisker = df[df>=lower_quartile-1.5*iqr].min() 
    return lower_whisker
     
def boxplot_sorted2(df):
    _, bp = pd.DataFrame.boxplot(df, return_type='both')
    whiskers = [whiskers.get_ydata() for whiskers in bp["whiskers"]]
    lw = [whisker[1] for whisker in whiskers]
    sorted_index = np.argsort(lw)
    return sorted_index, lw

def calc_lowerwhisker2(df):
    bp = pd.DataFrame.boxplot(df, return_type='dict')
    whiskers = [whiskers.get_ydata() for whiskers in bp["whiskers"]]
    lw = [whisker[1] for whisker in whiskers][::2]
    return lw

#%%
path = 'Z:/Projects/HTTK/'
#path = 'C:/Users/ppradeep/Desktop/HTTK/' 
#path = '/home/ppradeep/th034_home/Projects/HTTK/' 
#%%
## Read AR-ER data 
#AR data
AR_data = pd.read_excel(path+'ExternalPredictions/Supplemental File 2_ARpathway_Results_ConfScores_CI_2016-08-30.xlsx', index_col='CASRN')
AR_ACC_columns = [col for col in AR_data if col.endswith('ACC')]
AR_data_subset = AR_data[(AR_data['AUC.Agonist']>0.1) | (AR_data['AUC.Antagonist']>0.1)][AR_ACC_columns]

#%%
#ER data
ER_data = pd.read_excel(path+'ExternalPredictions/S2 ER SuperMatrix 2015-03-24.xlsx', index_col='CASRN')
ER_ACC_columns = [col for col in ER_data if col.endswith('ACC')]
ER_data_subset = ER_data[(ER_data['AUC.Agonist']>0.1) | (ER_data['AUC.Antagonist']>0.1)][ER_ACC_columns]

#%%
## Combine ER-AR data
ERARdata = pd.concat([AR_data_subset, ER_data_subset], axis = 1)
ERARdata.replace(1000000, np.nan, inplace = True)
ERARdata.to_csv(path+'ExternalPredictions/ER_AR_ACCData.csv')

# Fub, CLint predictions
ERAR_Fub_Clint_Props = pd.read_csv(path+'ExternalPredictions/ERAR_Fub_Clint_Props.csv') #Add index name in file
ERAR_Fub_Clint_Props.index = ERAR_Fub_Clint_Props['CASRN']

#%%
### File for Robert to get HTTK data (Css, Gastroplus predictions etc)
#ERAR_FubClintProps_ToPredictCss = ERAR_Fub_Clint_Props.loc[ERARdata.index,:].dropna()
#ERAR_FubClintProps_ToPredictCss.to_csv(path+'ExternalPredictions/ERAR_DataToPredictCss.txt', index = False)
##ER_AR_FubClintProps_ACC = ER_AR_FubClintProps_ACC[(ER_AR_FubClintProps_ACC['Medium Clearance Prediction'].notnull()) & (ER_AR_FubClintProps_ACC['ER_consensus_fub'].notnull())]

#%%
##################################################################################################################################################################
## Css - Toxcast and Expocast boxplot analysis
##################################################################################################################################################################

## HTTK package data from Robert Pearce after processing above generated data
#cssData = pd.read_csv(path+'ExternalPredictions/DataToPredictCss_with_css.csv', index_col='CASRN')['css'].dropna()
#cssData = pd.read_csv(path+'ExternalPredictions/DataToPredictCss32318.csv', index_col='CASRN').loc[:,['css', 'in_vitro']].dropna()
inSilicoCss = pd.read_csv(path+'ExternalPredictions/ERAR_analytic_css.csv', index_col='CASRN')['in.silico']
inVitroCss = pd.read_csv(path+'ExternalPredictions/ERAR_analytic_css.csv', index_col='CASRN')['in.vitro']

#%%
#Expocast data from John W.
expocastData = pd.read_excel(path+'ExternalPredictions/SupplementalTable1-070314.xlsx', index_col='CASRN').loc[inSilicoCss.index,:].dropna()
# Toxcast data with Css values
ERARdata_withCss = ERARdata.loc[inSilicoCss.index,:]

#%%
ERARdata_withInSilicoCss_Scaled = ERARdata_withCss.copy()
ERARdata_withInSilicoCss_ScaledError = ERARdata_withCss.copy()
ERARdata_withInVitroCss_Scaled = ERARdata_withCss.min(axis=1)

sigma = 0.77

for idx in ERARdata_withInSilicoCss_Scaled.index:
    ERARdata_withInSilicoCss_Scaled.loc[idx,:] = np.log10(ERARdata_withInSilicoCss_Scaled.loc[idx,:]/inSilicoCss[idx])
    ERARdata_withInSilicoCss_ScaledError.loc[idx,:] = np.log10(ERARdata_withInSilicoCss_ScaledError.loc[idx,:])-(np.log10(inSilicoCss[idx])+2*sigma)
    ERARdata_withInVitroCss_Scaled[idx] = np.log10(ERARdata_withInVitroCss_Scaled[idx]/inVitroCss[idx])

ERARdata_withInVitroCss_Scaled.dropna(inplace = True) 

#%%
ERARdata_withInSilicoCss_Scaled_Median = ERARdata_withInSilicoCss_Scaled.T.median()
expocastMedian = np.log10(expocastData['Total.median']) #column name = 0
expocastPerc = np.log10(expocastData['Total.95%-ile'])
ChemicalName = expocastData['ChemicalName']
ChemicalDTXSID = AR_data['DSSToxGSID']

#%%
y = pd.concat([ChemicalName, expocastMedian, expocastPerc, ERARdata_withInSilicoCss_Scaled_Median], axis = 1).dropna(how='any')#.head(10)
y.rename(columns = {0: 'ToxCast Median'}, inplace = True)

#%%
## Generate the boxplot
#plt.figure(figsize = (80,30), dpi = 200) #For supplemental
plt.figure(figsize = (45,20), dpi = 200) #For paper
ax = plt.axes()        
ax.xaxis.grid()

x = range(1,len(y)+1)

## Plot ERAR Toxcast Data
#index_sorted, lower_whisker_toxcast = boxplot_sorted2(ERARdata_withInSilicoCss_Scaled.loc[y.index,:].T)
index_sorted, lower_whisker_toxcast = boxplot_sorted(ERARdata_withInSilicoCss_Scaled.loc[y.index,:].T)
#p1 = plt.errorbar(-2, -2, yerr = 0, c = 'b', fmt = 'o', marker = 'o', lolims=1, label = 'ToxCast') #Force a boxplot legend for ToxCast data
p1 = plt.scatter(x, lower_whisker_toxcast, marker = 's', c = 'b', s = 50, label = None) ##ERAR Median value #c = color

xticks_suppl = y.loc[index_sorted, 'ChemicalName']
xticks_paper = y.loc[index_sorted].index
yticks = ['a', 'b']

## Plot the median ToxCast values
#p2 = plt.scatter(x, y.loc[index_sorted, 'ToxCast Median'], marker = 'o', c = 'b', label = None) ##ERAR Median value #c = color

# Plot the error in insilico Css Toxcast value
lower_whisker = calc_lowerwhisker(ERARdata_withInSilicoCss_ScaledError.loc[y.index,:].T)
p3 = plt.scatter(x, lower_whisker[index_sorted], marker = 's', c = 'k', s = 50, label = 'Lowest ToxCast + 2$\sigma_{residuals}$ using In silico Css ') 

# Plot the lowest in vitro Css Toxcast value
p4 = plt.scatter(x, ERARdata_withInVitroCss_Scaled[index_sorted], marker = 'o', c = 'r', s = 50, label = 'Lowest ToxCast using In vitro Css') 

#Plot expocast data
y_error_upper = y.loc[index_sorted,'Total.95%-ile']
yValue = y.loc[index_sorted, 'Total.median']
p5 = plt.errorbar(x, yValue, yerr = (y_error_upper-yValue), c = 'g', fmt = 'o', marker = 's', lolims=np.ones(len(y)), label = 'ExpoCast') ##Expocast data

#plt.xticks(x, xticks_suppl, rotation = 'vertical', fontsize = 21) #For supplemental
plt.xticks(x, x, rotation = 'vertical', fontsize = 12) #For paper
plt.yticks(fontsize = 22)
labels = ['Lowest ToxCast using In silico Css', 'Lowest ToxCast+2$\sigma_{residuals}$ using In silico Css', 'Lowest ToxCast using In vitro Css', 'ExpoCast']
plt.legend([p1, p3, p4, p5], labels,  fontsize = 30)
#plt.ylim([-0.01,0.75])
plt.xlim([-1,len(x)+1])
plt.ylabel('$Log_{10}$(Dose)', fontsize = 40, labelpad = 10)
plt.xlabel('Chemical', fontsize = 40, labelpad = 10)
plt.savefig(path+'ExternalPredictions/ERAR-boxplot_ForPaper_New.jpg', bbox_inches='tight')
#plt.savefig(path+'ExternalPredictions/ERAR-boxplot_ForSuppl.jpg', bbox_inches='tight')

#%%
df = ERARdata_withInSilicoCss_Scaled.loc[y.index,:].head().T
bp = pd.DataFrame.boxplot(df, return_type='dict')
whiskers = [whiskers.get_ydata() for whiskers in bp["whiskers"]]
lw = [whisker[1] for whisker in whiskers][::2]
    
    #%%
## Generate the output file for supplemental
SupplementalData = pd.concat([ChemicalDTXSID[y.index], y, lower_whisker_toxcast, ERARdata_withInVitroCss_Scaled[y.index], lower_whisker[y.index]], axis = 1)#.head(10)
SupplementalData.rename(columns  = {'Total.median': '1. ExpoCast Median', 'Total.95%-ile' : '2. ExpoCast Upper 95%-ile',\
                                    'ToxCast Median': '3. ToxCast Median', 0: '4: Lowest ToxCast + In Silico (lower whisker)', \
                                    1: '5. Lowest ToxCast + In vitro Css', \
                                    2: '6. Lowest ToxCast + In silico Css + 2sigma_residuals'}, inplace = True)
SupplementalData['Difference (2-4)'] = SupplementalData['2. ExpoCast Upper 95%-ile']-SupplementalData['4: Lowest ToxCast + In Silico (lower whisker)']
SupplementalData['Difference (2-5)'] = SupplementalData['2. ExpoCast Upper 95%-ile']-SupplementalData['5. Lowest ToxCast + In vitro Css']
SupplementalData['Difference (2-6)'] = SupplementalData['2. ExpoCast Upper 95%-ile']-SupplementalData['6. Lowest ToxCast + In silico Css + 2sigma_residuals']

SupplementalData.to_csv(path+'ExternalPredictions/Supplemental_ERAR-BERPlotData.csv', index_label='CASRN')
#%%
##################################################################################################################################################################
## Comparison with ADMET predictions
##################################################################################################################################################################

## Admet predictions
Sipes2017_Data = pd.read_csv(path+'ExternalPredictions/Sipes2017.csv', index_col='CAS')
Sipes2017_Data.rename(columns = {"Human.Funbound.plasma": "admet_Fub", "Human.Clint": "admet_Clint"}, inplace = True)
admet_Fub = Sipes2017_Data['admet_Fub']
admet_Clint = np.log10(Sipes2017_Data['admet_Clint']+0.001)

## Pradeep predictions
Pradeep_Fub = ERAR_Fub_Clint_Props['Predicted Fub']
Pradeep_Clint = np.log10(ERAR_Fub_Clint_Props['Clearance Prediction']+0.001)

#%%
## True: Read experimental Fub data from the 2 sources and combine them together
FubData1 = pd.read_csv(path+'data/Prachi-112117.txt', index_col='CAS').loc[:,['Human.Funbound.plasma', 'Human.Funbound.plasma.Reference']]
FubData2 = pd.read_excel(path+'data/AFFINITY_Model_Results-2018-02-27.xlsx', index_col = 'CAS')[['Fup.Med']]
FubData2.rename(columns = {'Fup.Med': 'Human.Funbound.plasma'}, inplace = True)
FubData2['Human.Funbound.plasma.Reference'] = 'EPA 2018'

casList = list(set(FubData1.index.tolist()+FubData2.index.tolist()))
True_Fub = pd.DataFrame(index = casList, columns = ['Human.Funbound.plasma', 'Human.Funbound.plasma.Reference'])
# Update the training data. If y value is available from later data (data 2 or 3) use that, if not use from old data (data1)
for cas in True_Fub.index:
    try:
        if cas in FubData1.index:
            True_Fub.loc[cas, 'Human.Funbound.plasma'] = FubData1.loc[cas,'Human.Funbound.plasma']
            True_Fub.loc[cas, 'Human.Funbound.plasma.Reference'] = FubData1.loc[cas, 'Human.Funbound.plasma.Reference']
        if cas in FubData2.index:
            True_Fub.loc[cas, 'Human.Funbound.plasma'] = FubData2.loc[cas,'Human.Funbound.plasma']
            True_Fub.loc[cas, 'Human.Funbound.plasma.Reference'] = FubData2.loc[cas, 'Human.Funbound.plasma.Reference']
    except:
        pass
True_Fub.dropna(inplace = True) #Retain data with y variable values

True_Fub[True_Fub==0] = 0.005 # Set 0 values to default limit of detection units
#%%
## True: Read experimental Clint data from the 2 sources and combine them together
ClintData1 = pd.read_csv(path+'data/Prachi-112117.txt', index_col = 'CAS').loc[:,['Human.Clint', 'Human.Clint.Reference']]
ClintData2 = pd.read_excel(path+'data/CLint-2018-03-01-Results.xlsx', index_col = 'CAS')[['CLint.1uM.Median']]
ClintData2.rename(columns = {'CLint.1uM.Median': 'Human.Clint'}, inplace = True)
ClintData2['Human.Clint.Reference'] = 'EPA 2018'

casList = list(set(ClintData1.index.tolist()+ClintData2.index.tolist()))
True_Clint = pd.DataFrame(index = casList, columns = ['Human.Clint', 'Human.Clint.Reference'])
# Update the training data. If y value is available from later data (data 2 or 3) use that, if not use from old data (data1)
for cas in True_Clint.index:
    try:
        if cas in ClintData1.index:
            True_Clint.loc[cas, 'Human.Clint'] = ClintData1.loc[cas,'Human.Clint']
            True_Clint.loc[cas, 'Human.Clint.Reference'] = ClintData1.loc[cas, 'Human.Clint.Reference']
        if cas in FubData2.index:
            True_Clint.loc[cas, 'Human.Clint'] = ClintData2.loc[cas,'Human.Clint']
            True_Clint.loc[cas, 'Human.Clint.Reference'] = ClintData2.loc[cas, 'Human.Clint.Reference']
    except:
        pass
True_Clint.dropna(inplace = True)

#%%
## Combine the true, admet, pradeep and Residuals from Admet prediction and Pradeep prediction
fub = pd.concat([True_Fub['Human.Funbound.plasma'], admet_Fub, Pradeep_Fub, True_Fub['Human.Funbound.plasma'].sub(admet_Fub), True_Fub['Human.Funbound.plasma'].sub(Pradeep_Fub)], axis = 1).dropna()
clint = pd.concat([True_Clint['Human.Clint'].apply(lambda x: np.log10(x+0.001)), admet_Clint, Pradeep_Clint, True_Clint['Human.Clint'].apply(lambda x: np.log10(x+0.001)).sub(admet_Clint), True_Clint['Human.Clint'].apply(lambda x: np.log10(x+0.001)).sub(Pradeep_Clint)], axis = 1).dropna()

fub.rename(columns={0: 'ADMET Residuals', 1: 'Pradeep Residuals'}, inplace = True)
clint.rename(columns={0: 'ADMET Residuals', 1: 'Pradeep Residuals'}, inplace = True)

#%%

#%%
# RMSE and R2 of ADMET Predictor and Pradeep et al versus true 
rmse_admetFub = round(np.sqrt(sm.mean_squared_error(fub['Human.Funbound.plasma'], fub['admet_Fub'])),2)
r2_admetFub = round(r2_score(fub['Human.Funbound.plasma'], fub['admet_Fub']),2)
rmse_ppradeepFub = round(np.sqrt(sm.mean_squared_error(fub['Human.Funbound.plasma'], fub['Predicted Fub'])),2)
r2_ppradeepFub = round(r2_score(fub['Human.Funbound.plasma'], fub['Predicted Fub']),2)

rmse_admetClint = round(np.sqrt(sm.mean_squared_error(clint['Human.Clint'], clint['admet_Clint'])),2)
r2_admetClint = round(r2_score(clint['Human.Clint'], clint['admet_Clint']),2)
rmse_ppradeepClint = round(np.sqrt(sm.mean_squared_error(clint['Human.Clint'], clint['Clearance Prediction'])),2)
r2_ppradeepClint = round(r2_score(clint['Human.Clint'], clint['Clearance Prediction']),2)

#%%
# RMSE and R2 of ADMET Predictor and Pradeep et al residuals (0: admet residual, 1: pradeep residual)
rmse_admetppradeep_Clint = round(np.sqrt(sm.mean_squared_error(clint['ADMET Residuals'], clint['Pradeep Residuals'])),2)
r2_admetppradeep_Clint = round(r2_score(clint['ADMET Residuals'], clint['Pradeep Residuals']),2)
rmse_admetppradeep_Fub = round(np.sqrt(sm.mean_squared_error(fub['ADMET Residuals'], fub['Pradeep Residuals'])),2)
r2_admetppradeep_Fub = round(r2_score(fub['ADMET Residuals'], fub['Pradeep Residuals']),2)

#%%
ax = fub.plot(x='ADMET Residuals', y='Pradeep Residuals', style='.', figsize = (12,12), markersize = 8)
ax.text(-0.95, 0.90, r'$N: %d$' %len(fub), fontsize = 18)
ax.text(-0.95, 0.80, r'$R^{2}$ (ADMET-Observed): $%.2f$' %r2_admetFub, fontsize = 20)
ax.text(-0.95, 0.73, r'$R^{2}$ (This work-Observed): $%.2f$' %r2_ppradeepFub, fontsize = 20)
ax.text(-0.95, 0.64, r'$RMSE$ (ADMET-Observed): $%.2f$' %rmse_admetFub, fontsize = 20)
ax.text(-0.95, 0.57, r'$RMSE$ (This work-Observed): $%.2f$' %rmse_ppradeepFub, fontsize = 20)

ax.text(-0.95, 0.45, r'$R^{2}$ (ADMET-This work): $%.2f$' %r2_admetppradeep_Fub, fontsize = 20)

plt.xlim([-1,1])
plt.ylim([-1,1])
plt.xticks(fontsize = 20, rotation = 'vertical')
plt.yticks(fontsize = 20)

axes = plt.gca()
ymin, ymax = axes.get_ylim()
xmin, xmax = axes.get_xlim()
# Draw a diagonal line
plt.plot([xmin, xmax], [ymin, ymax], color = 'k', alpha = 0.75)
# Mark the outlier area1
plt.axhline(y = 0.25, c = 'k', xmin = abs((xmin-(0.25))/(xmax-xmin)), linestyle=':')
plt.axvline(x = 0.25, c = 'k', ymin = abs((ymin-(0.25))/(ymax-ymin)), linestyle=':')
plt.axhline(y = -0.25, c = 'k', xmax = abs((xmin-(-0.25))/(xmax-xmin)), linestyle=':')
plt.axvline(x = -0.25, c = 'k', ymax = abs((ymin-(-0.25))/(ymax-ymin)), linestyle=':')
# Mark the outlier area2
plt.axhline(y = 0.5, c = 'k', xmin = abs((xmin-(0.5))/(xmax-xmin)), linestyle='--')
plt.axvline(x = 0.5, c = 'k', ymin = abs((ymin-(0.5))/(ymax-ymin)), linestyle='--')
plt.axhline(y = -0.5, c = 'k', xmax = abs((xmin-(-0.5))/(xmax-xmin)), linestyle='--')
plt.axvline(x = -0.5, c = 'k', ymax = abs((ymin-(-0.5))/(ymax-ymin)), linestyle='--')

plt.xlabel('Fraction Unbound Residuals (ADMET Predictor)', fontsize = 24, labelpad = 10)
plt.ylabel('Fraction Unbound Residuals (This work)', fontsize = 24, labelpad = 10)
plt.legend().set_visible(False)
plt.savefig(path+'ExternalPredictions/Fub-Residuals.jpg', bbox_inches='tight')
#%%
plt.figure(figsize=[12,8], dpi = 300)
admet_mean = fub['ADMET Residuals'].mean()
ppradeep_mean = fub['Pradeep Residuals'].mean()
fub['ADMET Residuals'].hist(label = 'ADMET ($\mu$ = %0.2f)' %admet_mean, alpha = 0.75)
fub['Pradeep Residuals'].hist(label = 'This work ($\mu$ = %0.2f)' %ppradeep_mean, color = 'g', alpha = 0.6)
plt.legend(fontsize = 18)
plt.xticks(fontsize = 24, rotation = 'vertical')
plt.yticks(fontsize = 24)
plt.xlabel('Residuals', size = 36, labelpad = 20)
plt.ylabel('Frequency', size = 36, labelpad = 20)
plt.savefig(path+'ExternalPredictions/ResidualsADMETThiswork_FubHist.png', bbox_inches='tight')

#%%
ax = clint.plot(x='ADMET Residuals', y='Pradeep Residuals', style='.', figsize = (12,12), markersize = 8)

ax.text(-6.75, 5.25, r'$N: %d$' %len(clint), fontsize = 18)
ax.text(-6.75, 4.50, r'$R^{2}$ (ADMET-Observed): $%.2f$' %r2_admetClint, fontsize = 20)
ax.text(-6.75, 4.10, r'$R^{2}$ (This work-Observed): $%.2f$' %r2_ppradeepClint, fontsize = 20)
ax.text(-6.75, 3.65, r'$RMSE$ (ADMET-Observed): $%.2f$' %rmse_admetClint, fontsize = 20)
ax.text(-6.75, 3.15, r'$RMSE$ (This work-Observed): $%.2f$' %rmse_ppradeepClint, fontsize = 20)

ax.text(-6.75, 2.45, r'$R^{2}$ (ADMET-This work): $%.2f$' %r2_admetppradeep_Clint, fontsize = 20)

plt.xlim([-7,6])
plt.ylim([-7,6])
plt.xticks(fontsize = 20, rotation = 'vertical')
plt.yticks(fontsize = 20)

axes = plt.gca()
ymin, ymax = axes.get_ylim()
xmin, xmax = axes.get_xlim()
# Draw a diagonal line
plt.plot([xmin, xmax], [ymin, ymax], color = 'k', alpha = 0.75)
# Mark the outlier area1
plt.axhline(y = -2, c = 'k', xmax = abs((xmin-(-2))/(xmax-xmin)), linestyle='--')
plt.axvline(x = -2, c = 'k', ymax = abs((ymin-(-2))/(ymax-ymin)),  linestyle='--')
# Mark the outlier area2
plt.axvline(x = 3, c = 'k', linestyle=':')

plt.xlabel('Intrinsic Clearance Residuals (ADMET Predictor)', fontsize = 24, labelpad = 10)
plt.ylabel('Intrinsic Clearance Residuals (This work)', fontsize = 24, labelpad = 10)
plt.legend().set_visible(False)
plt.savefig(path+'ExternalPredictions/Clint-Residuals.jpg', bbox_inches='tight')

#%%
plt.figure(figsize=[12,8], dpi = 300)
admet_mean = clint['ADMET Residuals'].mean()
ppradeep_mean = clint['Pradeep Residuals'].mean()
clint['ADMET Residuals'].hist(label = 'ADMET ($\mu$ = %0.2f)' %admet_mean, alpha = 0.75)
clint['Pradeep Residuals'].hist(label = 'This work ($\mu$ = %0.2f)' %ppradeep_mean, color = 'g', alpha = 0.6)
plt.legend(fontsize = 24)
plt.xticks(fontsize = 24, rotation = 'vertical')
plt.yticks(fontsize = 24)
plt.xlabel('Residuals', size = 36, labelpad = 20)
plt.ylabel('Frequency', size = 36, labelpad = 20)
plt.savefig(path+'ExternalPredictions/ResidualsADMETThiswork_ClintHist.png', bbox_inches='tight')

#%%
##################################################################################################################################################################
## Under/Overpredicted chemicals analysis
##################################################################################################################################################################
## Fub
fub_analysis_chemicals = Sipes2017_Data.loc[fub[(abs(fub[0])>0.250) & (abs(fub[1])>0.250)].index, 'Compound']
#fub_analysis_chemicals = Sipes2017_Data.loc[fub[(fub[0]>0.50) & (fub[1]>0.50)].index, 'Compound']
fub_analysis_chemicals_props = ERAR_Fub_Clint_Props.loc[fub_analysis_chemicals.index, ['h_pKa', 'h_pKb', 'h_logP', 'Molecular Weight']]
fub_analysis_chemicals_props['Molecular Weight'] = fub_analysis_chemicals_props['Molecular Weight']/100
fub_analysis_chemicals_props.rename(columns={'h_pKa': 'pKa_Outliers', 'h_pKb': 'pKb_Outliers', 'h_logP': 'logP_Outliers', 'Molecular Weight': 'MW_Outliers$*10^{-2}$'}, inplace = True)
ix=[i for i in fub.index if i not in fub_analysis_chemicals.index]  

others_props = ERAR_Fub_Clint_Props.loc[ix, ['h_pKa', 'h_pKb', 'h_logP', 'Molecular Weight']]
others_props['Molecular Weight'] = others_props['Molecular Weight']/100
others_props.rename(columns={'h_pKa': 'pKa_Others', 'h_pKb': 'pKb_Others', 'h_logP': 'logP_Others', 'Molecular Weight': 'MW_Others$*10^{-2}$'}, inplace = True)

allChem = pd.concat([fub_analysis_chemicals_props, others_props])

plt.figure(figsize = (12, 8), dpi = 300)
bp = allChem.boxplot(showfliers=False,return_type='dict') 
[[item.set_color('r') for item in bp['medians']] for key in bp.keys()]
[[item.set_linewidth(0.8) for item in bp['medians']] for key in bp.keys()]
plt.xticks(fontsize = 16, rotation = 'vertical')
plt.yticks(fontsize = 16)
plt.savefig(path+'ExternalPredictions/ERAR-FubOutlierAnalysis25.jpg', bbox_inches='tight')
pd.concat([fub_analysis_chemicals, True_Fub, admet_Fub, ERAR_Fub_Clint_Props], axis = 1).dropna().to_csv(path+'ExternalPredictions/ERAR-FubOutliers25.csv')

#%%
## Clint
clint_analysis_chemicals = Sipes2017_Data.loc[clint[(clint[0]<-2) & (clint[1]<-2)].index, 'Compound'] #Outliers1
#clint_analysis_chemicals = Sipes2017_Data.loc[clint[(clint[0]>3)].index, 'Compound'] #Outliers2
clint_analysis_chemicals_props = ERAR_Fub_Clint_Props.loc[clint_analysis_chemicals.index, ['h_pKa', 'h_pKb', 'h_logP', 'Molecular Weight']]
clint_analysis_chemicals_props['Molecular Weight'] = clint_analysis_chemicals_props['Molecular Weight']/100
clint_analysis_chemicals_props.rename(columns={'h_pKa': 'pKa_Outliers', 'h_pKb': 'pKb_Outliers', 'h_logP': 'logP_Outliers', 'Molecular Weight': 'MW_Outliers$*10^{-2}$'}, inplace = True)
ix=[i for i in fub.index if i not in clint_analysis_chemicals.index]  

others_props = ERAR_Fub_Clint_Props.loc[ix, ['h_pKa', 'h_pKb', 'h_logP', 'Molecular Weight']]
others_props['Molecular Weight'] = others_props['Molecular Weight']/100
others_props.rename(columns={'h_pKa': 'pKa_Others', 'h_pKb': 'pKb_Others', 'h_logP': 'logP_Others', 'Molecular Weight': 'MW_Others$*10^{-2}$'}, inplace = True)

allChem = pd.concat([clint_analysis_chemicals_props, others_props])

plt.figure(figsize = (12, 8), dpi = 300)
bp = allChem.boxplot(showfliers=False,return_type='dict') 
[[item.set_color('r') for item in bp['medians']] for key in bp.keys()]
[[item.set_linewidth(0.8) for item in bp['medians']] for key in bp.keys()]
plt.xticks(fontsize = 16, rotation = 'vertical')
plt.yticks(fontsize = 16)
plt.savefig(path+'ExternalPredictions/ERAR-ClintOutlierAnalysis2.jpg', bbox_inches='tight')

pd.concat([clint_analysis_chemicals, True_Clint, admet_Clint, ERAR_Fub_Clint_Props], axis = 1).dropna().to_csv(path+'ExternalPredictions/ERAR-ClintOutliers2.csv')

#%%
## Print the residuals file for all chemicals
pd.concat([Sipes2017_Data['Compound'], True_Clint, clint, ERAR_Fub_Clint_Props], axis = 1).dropna().to_csv(path+'ExternalPredictions/ERAR-ClintResiduals.csv')
pd.concat([Sipes2017_Data['Compound'], True_Fub, fub, ERAR_Fub_Clint_Props], axis = 1).dropna().to_csv(path+'ExternalPredictions/ERAR-FubResiduals.csv')