# -*- coding: utf-8 -*-
"""
Created on Mon Apr 09 15:09:37 2018

@author: ppradeep
"""

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import sklearn.metrics as sm
from sklearn.metrics import r2_score

#%%
def boxplot_sorted(df):
    upper_quartile = df.quantile(q=0.75)
    lower_quartile = df.quantile(q=0.25)
    iqr = upper_quartile - lower_quartile
    #upper_whisker = df[df<=upper_quartile+1.5*iqr].max()
    lower_whisker = df[df>=lower_quartile-1.5*iqr].min()
    sorted_index = lower_whisker.sort_values().index
    return df[sorted_index].boxplot(showfliers=False, return_type="axes"), sorted_index
     
#%%
path = 'Z:/Projects/HTTK/'
#path = 'C:/Users/ppradeep/Desktop/HTTK/' 

#%%
httk_data = pd.read_csv(path+'ExternalPredictions/httk_analytic_css.csv', index_col='CASRN')

httkpackagefubclint_httkprops_css = np.log10(httk_data['httk'])
httkpackagefubclint_moeprops_css = np.log10(httk_data['httk.cl.qsar.prop'])

pradeepqsarfubclint_httkprops_css = np.log10(httk_data['httk.prop.qsar.cl'])
pradeepqsarfubclint_moeprops_css = np.log10(httk_data['overwritten.httk'])

#%%

fig, axarr = plt.subplots(3,2,figsize=(8, 10), dpi = 300, sharex = True, sharey = True)

#####  1 ##### 
axarr[0,0].scatter(httkpackagefubclint_httkprops_css, httkpackagefubclint_moeprops_css, c = 'g', alpha = 0.75, s = 10)
axarr[0,0].plot([0, 1], [0, 1], transform=axarr[0,0].transAxes, c = 'k', alpha = 0.5)
xlim = axarr[0,0].get_xlim()
ylim = axarr[0,0].get_ylim()
axarr[0,0].plot([xlim[0], xlim[1]], [ylim[0]+2, ylim[1]+2], c = 'r', linestyle = '--', alpha = 0.5)
axarr[0,0].plot([xlim[0]+2, xlim[1]+2], [ylim[0], ylim[1]], c = 'r', linestyle = '--', alpha = 0.5)
axarr[0,0].set_xlim(xlim)
axarr[0,0].set_ylim(ylim)

residual = httkpackagefubclint_moeprops_css - pradeepqsarfubclint_moeprops_css
mean = residual.mean()
sigma = residual.std()

rmse = np.sqrt(sm.mean_squared_error(httkpackagefubclint_httkprops_css, httkpackagefubclint_moeprops_css))
r2 = r2_score(httkpackagefubclint_httkprops_css, httkpackagefubclint_moeprops_css)

axarr[0,0].annotate('$\sigma_{Residuals}$: %0.2f' %sigma,  [-3.8,5], fontsize=12)
axarr[0,0].annotate('$RMSE$: %0.2f' %rmse,  [-3.8,4], fontsize=12)
axarr[0,0].annotate('$R^{2}$: %0.2f' %r2,  [-3.8,3], fontsize=12)


axarr[0,0].set(xlabel='Css (In vitro) | PhysChem (HTTK) \n (a)', ylabel='Css (In vitro) | PhysChem (MOE)')

#####  2 ##### 
axarr[0,1].scatter(pradeepqsarfubclint_httkprops_css, pradeepqsarfubclint_moeprops_css, c = 'g', alpha = 0.75, s = 10)
axarr[0,1].plot([0, 1], [0, 1], transform=axarr[0,1].transAxes, c = 'k', alpha = 0.5)
xlim = axarr[0,1].get_xlim()
ylim = axarr[0,1].get_ylim()
axarr[0,1].plot([xlim[0], xlim[1]], [ylim[0]+2, ylim[1]+2], c = 'r', linestyle = '--', alpha = 0.5)
axarr[0,1].plot([xlim[0]+2, xlim[1]+2], [ylim[0], ylim[1]], c = 'r', linestyle = '--', alpha = 0.5)
axarr[0,1].set_xlim(xlim)
axarr[0,1].set_ylim(ylim)

residual = pradeepqsarfubclint_httkprops_css - pradeepqsarfubclint_moeprops_css
mean = residual.mean()
sigma = residual.std()

rmse = np.sqrt(sm.mean_squared_error(pradeepqsarfubclint_httkprops_css, pradeepqsarfubclint_moeprops_css))
r2 = r2_score(pradeepqsarfubclint_httkprops_css, pradeepqsarfubclint_moeprops_css)

axarr[0,1].annotate('$\sigma_{Residuals}$: %0.2f' %sigma,  [-3.8,5], fontsize=12)
axarr[0,1].annotate('$RMSE$: %0.2f' %rmse,  [-3.8,4], fontsize=12)
axarr[0,1].annotate('$R^{2}$: %0.2f' %r2,  [-3.8,3], fontsize=12)

axarr[0,1].set(xlabel='Css (In silico) | PhysChem (HTTK) \n (b)', ylabel='Css (In silico) | PhysChem (MOE)')

##### 3##### 
axarr[1,0].scatter(httkpackagefubclint_httkprops_css, pradeepqsarfubclint_httkprops_css, alpha = 0.75, s = 10)
axarr[1,0].plot([0, 1], [0, 1], transform=axarr[1,0].transAxes, c = 'k', alpha = 0.5)
xlim = axarr[1,0].get_xlim()
ylim = axarr[1,0].get_ylim()
axarr[1,0].plot([xlim[0], xlim[1]], [ylim[0]+2, ylim[1]+2], c = 'r', linestyle = '--', alpha = 0.5)
axarr[1,0].plot([xlim[0]+2, xlim[1]+2], [ylim[0], ylim[1]], c = 'r', linestyle = '--', alpha = 0.5)
axarr[1,0].set_xlim(xlim)
axarr[1,0].set_ylim(ylim)

residual = httkpackagefubclint_httkprops_css - pradeepqsarfubclint_httkprops_css
mean = residual.mean()
sigma = residual.std()

rmse = np.sqrt(sm.mean_squared_error(httkpackagefubclint_httkprops_css, pradeepqsarfubclint_httkprops_css))
r2 = r2_score(httkpackagefubclint_httkprops_css, pradeepqsarfubclint_httkprops_css)

axarr[1,0].annotate('$\sigma_{Residuals}$: %0.2f' %sigma,  [-3.8,5], fontsize=12)
axarr[1,0].annotate('$RMSE$: %0.2f' %rmse,  [-3.8,4], fontsize=12)
axarr[1,0].annotate('$R^{2}$: %0.2f' %r2,  [-3.8,3], fontsize=12)

axarr[1,0].set(xlabel='Css (In vitro) | PhysChem (HTTK) \n (c)', ylabel='Css (In silico) | PhysChem (HTTK)')

#####  4 ##### 
axarr[1,1].scatter(httkpackagefubclint_moeprops_css, pradeepqsarfubclint_moeprops_css, alpha = 0.75, s = 10)
axarr[1,1].plot([0, 1], [0, 1], transform=axarr[1,1].transAxes, c = 'k', alpha = 0.5)
xlim = axarr[1,1].get_xlim()
ylim = axarr[1,1].get_ylim()
axarr[1,1].plot([xlim[0], xlim[1]], [ylim[0]+2, ylim[1]+2], c = 'r', linestyle = '--', alpha = 0.5)
axarr[1,1].plot([xlim[0]+2, xlim[1]+2], [ylim[0], ylim[1]], c = 'r', linestyle = '--', alpha = 0.5)
axarr[1,1].set_xlim(xlim)
axarr[1,1].set_ylim(ylim)

residual = httkpackagefubclint_moeprops_css - pradeepqsarfubclint_moeprops_css
mean = residual.mean()
sigma = residual.std()

rmse = np.sqrt(sm.mean_squared_error(httkpackagefubclint_moeprops_css, pradeepqsarfubclint_moeprops_css))
r2 = r2_score(httkpackagefubclint_moeprops_css, pradeepqsarfubclint_moeprops_css)

axarr[1,1].annotate('$\sigma_{Residuals}$: %0.2f' %sigma,  [-3.8,5], fontsize=12)
axarr[1,1].annotate('$RMSE$: %0.2f' %rmse,  [-3.8,4], fontsize=12)
axarr[1,1].annotate('$R^{2}$: %0.2f' %r2,  [-3.8,3], fontsize=12)

axarr[1,1].set(xlabel='Css (In vitro) | PhysChem (MOE) \n (d)', ylabel='Css (In silico) | PhysChem (MOE)')

#####  5 ##### 
axarr[2,0].scatter(httkpackagefubclint_httkprops_css, pradeepqsarfubclint_moeprops_css, alpha = 0.75, s = 10)
axarr[2,0].plot([0, 1], [0, 1], transform=axarr[2,0].transAxes, c = 'k', alpha = 0.5)
xlim = axarr[2,0].get_xlim()
ylim = axarr[2,0].get_ylim()
axarr[2,0].plot([xlim[0], xlim[1]], [ylim[0]+2, ylim[1]+2], c = 'r', linestyle = '--', alpha = 0.5)
axarr[2,0].plot([xlim[0]+2, xlim[1]+2], [ylim[0], ylim[1]], c = 'r', linestyle = '--', alpha = 0.5)
axarr[2,0].set_xlim(xlim)
axarr[2,0].set_ylim(ylim)

residual = httkpackagefubclint_httkprops_css - pradeepqsarfubclint_moeprops_css
mean = residual.mean()
sigma = residual.std()

rmse = np.sqrt(sm.mean_squared_error(httkpackagefubclint_httkprops_css, pradeepqsarfubclint_moeprops_css))
r2 = r2_score(httkpackagefubclint_httkprops_css, pradeepqsarfubclint_moeprops_css)

axarr[2,0].annotate('$\sigma_{Residuals}$: %0.2f' %sigma,  [-3.8,5], fontsize=12)
axarr[2,0].annotate('$RMSE$: %0.2f' %rmse,  [-3.8,4], fontsize=12)
axarr[2,0].annotate('$R^{2}$: %0.2f' %r2,  [-3.8,3], fontsize=12)

axarr[2,0].set(xlabel='Css (In vitro) | PhysChem (HTTK) \n (e)', ylabel='Css (In silico) | PhysChem (MOE)')

#####  6 ##### 
axarr[2,1].scatter(httkpackagefubclint_moeprops_css, pradeepqsarfubclint_httkprops_css, alpha = 0.75, s = 10)
axarr[2,1].plot([0, 1], [0, 1], transform=axarr[2,1].transAxes, c = 'k', alpha = 0.5)
xlim = axarr[2,1].get_xlim()
ylim = axarr[2,1].get_ylim()
axarr[2,1].plot([xlim[0], xlim[1]], [ylim[0]+2, ylim[1]+2], c = 'r', linestyle = '--', alpha = 0.5)
axarr[2,1].plot([xlim[0]+2, xlim[1]+2], [ylim[0], ylim[1]], c = 'r', linestyle = '--', alpha = 0.5)
axarr[2,1].set_xlim(xlim)
axarr[2,1].set_ylim(ylim)

residual = httkpackagefubclint_moeprops_css - pradeepqsarfubclint_httkprops_css
mean = residual.mean()
sigma = residual.std()

rmse = np.sqrt(sm.mean_squared_error(httkpackagefubclint_moeprops_css, pradeepqsarfubclint_httkprops_css))
r2 = r2_score(httkpackagefubclint_moeprops_css, pradeepqsarfubclint_httkprops_css)

axarr[2,1].annotate('$\sigma_{Residuals}$: %0.2f' %sigma,  [-3.8,5], fontsize=12)
axarr[2,1].annotate('$RMSE$: %0.2f' %rmse,  [-3.8,4], fontsize=12)
axarr[2,1].annotate('$R^{2}$: %0.2f' %r2,  [-3.8,3], fontsize=12)

axarr[2,1].set(xlabel='Css (In vitro) | PhysChem (MOE) \n (f)', ylabel='Css (In silico) | PhysChem (HTTK)')

plt.savefig(path+'ExternalPredictions/httkCssPlots.png', bbox_inches='tight')

#%%
# Residual 
