# -*- coding: utf-8 -*-
"""
Created on Wed Mar 08 12:16:35 2017

@author: ppradeep
"""

import os
clear = lambda: os.system('cls')
clear()

## Import packages
import warnings
warnings.filterwarnings('ignore')

import pandas as pd
import numpy as np

# Classifiers
from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier
from sklearn import svm, preprocessing

path = 'C:/Users/ppradeep/Desktop/HTTK/' 
path = 'Z:/Projects/HTTK/' 

#%%
# Normalize descriptors: Transform variables to mean=0, variance=1
def normalizeDescriptors(X):
    x_norm = preprocessing.scale(X)
    x_norm = pd.DataFrame(x_norm, index = X.index) 
    x_norm.columns = X.columns
    return x_norm
#%%
###########################################################################
###########################################################################
## Build the final models
###########################################################################
###########################################################################

###########################################################################
## Read training data 

data1 = pd.read_csv(path+'data/Prachi-112117.txt', index_col = 'CAS').loc[:,['All.Compound.Names', 'Human.Funbound.plasma', 'Human.Clint']]
data1.rename(columns={'All.Compound.Names' : 'Name'}, inplace = True)
data2 = pd.read_excel(path+'data/AFFINITY_Model_Results-2018-02-27.xlsx', index_col = 'CAS').loc[:,['Name','Fup.Med']]
data2.rename(columns={'Name': 'All.Compound.Names','Fup.Med':'Human.Funbound.plasma'}, inplace = True)
data3 = pd.read_excel(path+'data/CLint-2018-03-01-Results.xlsx', index_col = 'CAS').loc[:,['Name','CLint.1uM.Median']]
data3.rename(columns={'Name': 'All.Compound.Names','CLint.1uM.Median':'Human.Clint'}, inplace = True)

#%%
###########################################################################
## Read training fingerprints

## Chemotyper FPs: 779 Toxprints 
df_chemotypes = pd.read_csv(path+'data/toxprint.txt', sep = ';', index_col='M_NAME') #Rename 'M_NAME' to 'CAS' in data file
## PubChem FPs: 881 bits
df_pubchem = pd.read_csv(path+'data/pubchem.txt', index_col='row ID')

#%%
###########################################################################
## Read continuous descriptors 

## MOE descriptors
df_moe = pd.read_csv(path+'data/moe.txt', index_col='CAS').dropna() #In MOE: Right click on mol -> Name -> Extract -> new field 'CAS'
moe = normalizeDescriptors(df_moe)
## PADEL descriptors
df_padel = pd.read_csv(path+'data/padel.txt', index_col='Name').dropna() 
padel = normalizeDescriptors(df_padel)
## CDK descriptors
df_cdk = pd.read_csv(path+'data/cdk.txt', index_col='row ID').dropna() #Add CAS column to file
cdk = normalizeDescriptors(df_cdk)

#%%
###########################################################################
## Features from the 5-fold CV model

fub_features = pd.read_csv(path+'output/Human.Funbound.plasma_Features.csv')
clint_features_clas = pd.read_csv(path+'output/Clint_Features_Classification.csv')
clint_features_reg = pd.read_csv(path+'output/Clint_Features_Regression.csv')

#%%
###########################################################################
## Model for Fraction Unbound in Plasma
###########################################################################

y_var = 'Human.Funbound.plasma'
# Create a new dataframe with chemical names and y variable value based on raw data
casList = list(set(data1.index.tolist()+data2.index.tolist()+data3.index.tolist()))
data = pd.DataFrame(index = casList, columns = ['Name',y_var])

# Update the training data. If y value is available from later data (data 2 or 3) use that, if not use from old data (data1)
for cas in data.index:
    try:
        if cas in data1.index:
            data.loc[cas,'Name'] = data1.loc[cas,'Name']
            data.loc[cas,y_var] = data1.loc[cas,y_var]
        if cas in data2.index:
            data.loc[cas,'Name'] = data2.loc[cas,'Name']
            data.loc[cas,y_var] = data2.loc[cas,y_var]
    except:
        pass
data.dropna(inplace = True) #Retain data with y variable values

## Extract y data
Y = data[y_var]
## Set data for modeling
## Transform Y
Y[Y==1.0] = 0.99
Y[Y==0] = 0.005
Y_model = (1-Y)/Y
Y_model = Y_model.apply(lambda x: np.log10(x))
Y_index = Y_model.index

#%%
## Combine fingerprints and perform feature selection
fingerprints = pd.concat([df_pubchem, df_chemotypes], axis=1).dropna()
fingerprints = fingerprints.loc[Y_index,:].dropna()
# Select fingerprints from the feature file
retain = [str(val.replace("'", "").replace(" ", "")) for val in fub_features.ix[0,'Fingerprints'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("-]",'-') ##manually check the last entry and correct it
fingerprints = fingerprints.loc[:,retain]

#%%
descriptors = pd.concat([padel, cdk], axis=1).dropna()
descriptors = descriptors.loc[Y_index,:].dropna()

# Select descriptors from the feature file
retain = [str(val.replace("'", "").replace(" ", "")) for val in fub_features.ix[0,'Padel+CDK'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]",'')
descriptors = descriptors.loc[:,retain]

#%%
## Combine all the descriptors and set the X and Y for training the model
data = pd.concat([Y_model, fingerprints, moe, descriptors], axis=1).dropna(axis=0, how='any')
X_fub_model = data.ix[:, data.columns != y_var]
Y_fub_model = data[y_var]

#%%
###########################################################################
## Develop model 

clf_fub1 = svm.SVR(epsilon = 0.1, C = 10, gamma = 0.01, kernel = "rbf")
clf_fub1 = clf_fub1.fit(X = X_fub_model, y = Y_fub_model)

clf_fub2 = RandomForestRegressor(max_features = 'auto', n_estimators = 500, random_state = 5)
clf_fub2 = clf_fub2.fit(X = X_fub_model, y = Y_fub_model)

#%%
###########################################################################
## Models for Intrinsic Clearance 
###########################################################################

## Set data for modeling
y_var = 'Human.Clint'
# Create a new dataframe with chemical names and y variable value based on raw data
casList = list(set(data1.index.tolist()+data2.index.tolist()+data3.index.tolist()))
data = pd.DataFrame(index = casList, columns = ['Name',y_var])
# Update the training data. If y value is available from later data (data 2 or 3) use that, if not use from old data (data1)
for cas in data.index:
    try:
        if cas in data1.index:
            data.loc[cas,'Name'] = data1.loc[cas,'Name']
            data.loc[cas,y_var] = data1.loc[cas,y_var]
        if cas in data2.index:
            data.loc[cas,'Name'] = data2.loc[cas,'Name']
            data.loc[cas,y_var] = data2.loc[cas,y_var]
    except:
        pass
data.dropna(inplace = True) #Retain data with y variable values
## Extract y data
Y = data[y_var]
## Transform the data: Bin the clearance variable for classification
Y_clas = Y.copy()
[Y_clas.set_value(idx, int(-3)) for idx in Y_clas[Y_clas <= 0.9].index] 
[Y_clas.set_value(idx, int(-2)) for idx in Y_clas[(Y_clas > 0.9) & (Y_clas <= 50)].index] 
[Y_clas.set_value(idx, int(-1)) for idx in Y_clas[Y_clas > 50].index] 
Y_clas = pd.Series(Y_clas, index = Y.index) 

#%%
low_median = Y[Y_clas[Y_clas==-3].index].median()
high_median = Y[Y_clas[Y_clas==-1].index].median()

#%%
###########################################################################
## Classification:

## Combine fingerprints and perform feature selection
fingerprints = pd.concat([df_pubchem, df_chemotypes], axis=1).dropna()
fingerprints = fingerprints.loc[Y_clas.index,:].dropna()
# Select fingerprints from the feature file
retain = [str(val.replace("'", "").replace(" ", "")) for val in clint_features_clas.ix[0,'Fingerprints'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]",'')
fingerprints = fingerprints.loc[:,retain]

##%%
#descriptors = pd.concat([padel, cdk], axis=1).dropna()
#descriptors = descriptors.loc[Y_clas.index,:].dropna()
#
## Select descriptors from the feature file
#retain = [str(val.replace("'", "").replace(" ", "")) for val in clint_features_clas.ix[0,'Padel+CDK'].split(',')]
#retain[0] = retain[0].replace("[", "")
#retain[len(retain)-1] = retain[len(retain)-1].replace("]",'')
#descriptors = descriptors.loc[:,retain]

#%%
## Classification: Combine all the descriptors and set the X and Y for training the classification model
data = pd.concat([Y_clas, fingerprints, moe], axis=1).dropna(axis=0, how='any')
X_ClintClas_model = data.ix[:, data.columns != y_var]
Y_ClintClas_model = data[y_var]

###########################################################################
## Develop classification model 

## Classification model
clf_clintClas = RandomForestClassifier(max_features = 'sqrt', n_estimators = 100, random_state = 5)
clf_clintClas = clf_clintClas.fit(X = X_ClintClas_model, y = Y_ClintClas_model.values.tolist())

###########################################################################
## Intrinsic Clearance Regression
###########################################################################
## Regression
## Extract y data for regression
Y_reg = Y[(Y > 0.9) & (Y <= 50)]
## Transform Y
Y_reg = Y_reg.apply(lambda x: np.log10(x))

#%%
## Combine fingerprints and perform feature selection
fingerprints = pd.concat([df_pubchem, df_chemotypes], axis=1).dropna()
fingerprints = fingerprints.loc[Y_reg.index,:].dropna()

# Select fingerprints from the feature file
retain = [str(val.replace("'", "").replace(" ", "")) for val in clint_features_reg.ix[0,'Fingerprints'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]",'')
fingerprints = fingerprints.loc[:,retain]

#%%
## Combine all the descriptors and set the X and Y for training the regression model
data = pd.concat([Y_reg, fingerprints, moe], axis=1).dropna(axis=0, how='any')

X_ClintReg_model = data.ix[:, data.columns != y_var]
Y_ClintReg_model = data[y_var]

#%%
###########################################################################
## Develop model 

clf_clintReg = RandomForestRegressor(max_features = 'auto', n_estimators = 500, random_state = 5)
clf_clintReg = clf_clintReg.fit(X = X_ClintReg_model, y = Y_ClintReg_model)


#%%
###########################################################################
## Predict external data
###########################################################################

###########################################################################
## ERAR Data

# ER data
ER_pubchem = pd.read_csv(path+'ExternalPredictions/pubchem_ER.csv', index_col='row ID')
ER_chemotypes = pd.read_csv(path+'ExternalPredictions/toxprint_v2_vs_ER.csv', index_col='M_NAME')
ER_moe = pd.read_csv(path+'ExternalPredictions/moe_ER.txt', index_col='CASRN')
ER_cdk = pd.read_csv(path+'ExternalPredictions/cdk_ER.csv', index_col='row ID') 
ER_padel = pd.read_csv(path+'ExternalPredictions/padel_ER.csv', index_col='Name').dropna()
# AR data 
AR_pubchem = pd.read_csv(path+'ExternalPredictions/pubchem_AR.csv', index_col='row ID') 
AR_chemotypes = pd.read_csv(path+'ExternalPredictions/toxprint_v2_vs_AR.csv', index_col='M_NAME')
AR_moe = pd.read_csv(path+'ExternalPredictions/moe_AR.txt', index_col='CASRN')
AR_cdk = pd.read_csv(path+'ExternalPredictions/cdk_AR.csv', index_col='row ID') 
AR_padel = pd.read_csv(path+'ExternalPredictions/padel_AR.csv', index_col='Name').dropna() 

# Combine ER and ER list of chemicals and features
ERAR_pubchem = pd.concat([AR_pubchem, ER_pubchem])
ERAR_pubchem = ERAR_pubchem.groupby(ERAR_pubchem.index).first()

ERAR_chemotypes = pd.concat([AR_chemotypes, ER_chemotypes])#.index.drop_duplicates()
ERAR_chemotypes = ERAR_chemotypes.groupby(ERAR_chemotypes.index).first()

ERAR_moe = pd.concat([AR_moe, ER_moe])
ERAR_moe = ERAR_moe.groupby(ERAR_moe.index).first()
ERARmoe = normalizeDescriptors(ERAR_moe)

ERAR_cdk = pd.concat([AR_cdk, ER_cdk])
ERAR_cdk = ERAR_cdk.groupby(ERAR_cdk.index).first()
ERARcdk = normalizeDescriptors(ERAR_cdk)

ERAR_padel = pd.concat([AR_padel, ER_padel])
ERAR_padel = ERAR_padel.groupby(ERAR_padel.index).first()
ERARpadel = normalizeDescriptors(ERAR_padel)

#%%
###########################################################################
## Fub prediction

## Set feature set (X) for ERAR data
ERAR_x = pd.concat([ERAR_pubchem, ERAR_chemotypes, ERARmoe, ERARcdk, ERARpadel], axis = 1).dropna().loc[:,X_fub_model.columns]#.dropna()

ERAR_pred1 = clf_fub1.predict(ERAR_x)
ERAR_pred2 = clf_fub2.predict(ERAR_x)
ERAR_consensus = (ERAR_pred1 + ERAR_pred2)/2
## Transform back to 0-1 range
ERAR_consensus_fub = 1/(1+np.exp(ERAR_consensus))
## Set the upper limit as 1
for idx, val in enumerate(ERAR_consensus_fub):
    if val > 1:
        ERAR_consensus_fub[idx] = 0.99
ERAR_predictions_fub = pd.DataFrame({'ERAR_PredictionsSVR': ERAR_pred1, 'ERAR_PredictionsRF': ERAR_pred2, 'ERAR_consensus': ERAR_consensus, 'Predicted Fub': ERAR_consensus_fub}, index = ERAR_x.index)
#ERAR_predictions_fub.to_csv(path+'ExternalPredictions/ERAR_Fub_predictions.csv')

#%%
###########################################################################
## Clint prediction

## Classification
## Set feature set (X) for ERAR data
ERAR_x = pd.concat([ERAR_pubchem, ERAR_chemotypes, ERARmoe], axis=1).dropna().ix[:,X_ClintClas_model.columns]
ERAR_pred_ClintClas = clf_clintClas.predict(ERAR_x)

## Set the values as low, medium, high
ERAR_pred_ClintBin = []
ERAR_medium_idx = [] # Index of medium classified external data
for idx, val in enumerate(ERAR_pred_ClintClas):
    if val == -3:
        ERAR_pred_ClintBin.append('Low')
    if val == -2:
        ERAR_pred_ClintBin.append('Medium')
        ERAR_medium_idx.append(idx)
    if val == -1:
        ERAR_pred_ClintBin.append('High')      

ERAR_pred_ClintBin = pd.Series(ERAR_pred_ClintBin, index = ERAR_x.index)
#%%
## Regression
## Set feature set (X) for ER data
ERAR_x = pd.concat([ERAR_pubchem, ERAR_chemotypes, ERARmoe], axis=1).dropna().ix[ERAR_medium_idx,X_ClintReg_model.columns]
ERAR_pred_ClintReg = clf_clintReg.predict(ERAR_x)

## Transform back to 0-1 range
ERAR_pred_ClintReg = np.exp(ERAR_pred_ClintReg)
for idx, val in enumerate(ERAR_pred_ClintReg):
    if val > 50:
        ERAR_pred_ClintReg[idx] = 50
        
ERAR_pred_ClintReg = pd.Series(ERAR_pred_ClintReg, index = ERAR_x.index)   

#%%
ERAR_predictions_Clint = pd.DataFrame({'Clearance Bin': ERAR_pred_ClintBin, 'Clearance Prediction': ERAR_pred_ClintReg})
for idx in ERAR_predictions_Clint[ERAR_predictions_Clint['Clearance Bin']=='Low'].index:
    ERAR_predictions_Clint.loc[idx, 'Clearance Prediction'] = low_median
for idx in ERAR_predictions_Clint[ERAR_predictions_Clint['Clearance Bin']=='High'].index:
    ERAR_predictions_Clint.loc[idx, 'Clearance Prediction'] = high_median    
#%%
#ERAR_predictions_Clint.to_csv(path+'ExternalPredictions/ERAR_Clint_predictions.txt')

#%%
## Final Output File
ERAR_Fub_Clint_Props = pd.concat([ERAR_predictions_fub['Predicted Fub'], ERAR_predictions_Clint['Clearance Bin'], ERAR_predictions_Clint['Clearance Prediction'], ERAR_moe, ERAR_cdk['Molecular Weight']], axis = 1)
ERAR_Fub_Clint_Props.to_csv(path+'ExternalPredictions/ERAR_Fub_Clint_Props.txt')

#%%
###########################################################################
## HTTK package data (chemicals with experimental Fub and Clint <=> in vitro Css values)
## These chemicals are a subset of the training set, so I can get the descriptors from there.

httk_pubchem = pd.read_csv(path+'ExternalPredictions/httk_chems_pubchem.csv', index_col='row ID') 
httk_chemotypes = pd.read_csv(path+'ExternalPredictions/toxprint_v2_vs_httk_chems.csv', index_col='M_NAME')
httk_moe = pd.read_csv(path+'ExternalPredictions/httk_chems_moe.txt', index_col='CASRN')
httk_cdk = pd.read_csv(path+'ExternalPredictions/httk_chems_cdk.csv', index_col='row ID') 
httk_padel = pd.read_csv(path+'ExternalPredictions/httk_chems_padel.csv', index_col='Name').dropna() 

#%%
moe = normalizeDescriptors(httk_moe)
padel = normalizeDescriptors(httk_padel)
cdk = normalizeDescriptors(httk_cdk)

#%%
###########################################################################
## Fub prediction

## Set feature set (X) 
httk_x = pd.concat([httk_pubchem, httk_chemotypes, moe, cdk, padel], axis = 1).dropna().loc[:,X_fub_model.columns]#.dropna()
httk_pred1 = clf_fub1.predict(httk_x)
httk_pred2 = clf_fub2.predict(httk_x)
httk_consensus = (httk_pred1 + httk_pred2)/2
## Transform back to 0-1 range
httk_consensus_fub = 1/(1+np.exp(httk_consensus))
## Set the upper limit as 1
for idx, val in enumerate(httk_consensus_fub):
    if val > 1:
        httk_consensus_fub[idx] = 1
httk_predictions_fub = pd.DataFrame({'httk_PredictionsSVR': httk_pred1, 'httk_PredictionsRF': httk_pred2, 'httk_consensus': httk_consensus, 'Predicted Fub': httk_consensus_fub}, index = httk_x.index)
#httk_predictions_fub.to_csv(path+'ExternalPredictions/httk_Fub_predictions.csv')

###########################################################################
## Clint prediction

## Classification
## Set feature set (X) 
httk_x = pd.concat([httk_pubchem, httk_chemotypes, moe], axis=1).dropna().ix[:,X_ClintClas_model.columns]
httk_pred_ClintClas = clf_clintClas.predict(httk_x)

## Set the values as low, medium, high
httk_pred_ClintBin = []
httk_medium_idx = [] # Index of medium classified external data
for idx, val in enumerate(httk_pred_ClintClas):
    if val == -3:
        httk_pred_ClintBin.append('Low')
    if val == -2:
        httk_pred_ClintBin.append('Medium')
        httk_medium_idx.append(idx)
    if val == -1:
        httk_pred_ClintBin.append('High')      

httk_pred_ClintBin = pd.Series(httk_pred_ClintBin, index = httk_x.index)

## Regression
## Set feature set (X) 
httk_x = pd.concat([httk_pubchem, httk_chemotypes, moe], axis = 1).dropna().ix[httk_medium_idx, X_ClintReg_model.columns]
httk_pred_ClintReg = clf_clintReg.predict(httk_x)

## Transform back to 0-1 range
httk_pred_ClintReg = np.exp(httk_pred_ClintReg)
for idx, val in enumerate(httk_pred_ClintReg):
    if val > 50:
        httk_pred_ClintReg[idx] = 50
        
httk_pred_ClintReg = pd.Series(httk_pred_ClintReg, index = httk_x.index)   
#%%
httk_predictions_Clint = pd.DataFrame({'Clearance Bin': httk_pred_ClintBin, 'Clearance Prediction': httk_pred_ClintReg})
for idx in httk_predictions_Clint[httk_predictions_Clint['Clearance Bin']=='Low'].index:
    httk_predictions_Clint.loc[idx, 'Clearance Prediction'] = low_median
for idx in httk_predictions_Clint[httk_predictions_Clint['Clearance Bin']=='High'].index:
    httk_predictions_Clint.loc[idx, 'Clearance Prediction'] = high_median    
    
## Final Output File
httk_Fub_Clint_Props = pd.concat([httk_predictions_fub['Predicted Fub'], httk_predictions_Clint['Clearance Bin'], httk_predictions_Clint['Clearance Prediction'], httk_moe, httk_cdk['Molecular Weight']], axis = 1)
httk_Fub_Clint_Props.to_csv(path+'ExternalPredictions/httk_Fub_Clint_Props.txt')
